@@ -54,12 +54,13 @@ typedef struct {
 
 extern int UdmOpenCache(UDM_ENV * Env, char *errmsg);
 extern void UdmCloseCache(UDM_ENV  * Env);
+extern int UdmClearCacheTree(UDM_ENV * Env);
+extern int UdmSplitCacheLog(UDM_ENV * Env,int log);
+extern int UdmPreSplitCacheLog(UDM_ENV * Env);
+
 
 extern int UdmStoreWordsCache(UDM_AGENT *Indexer,int url_id,int site_id,const char *category,const char * tag);
 extern UDM_SEARCHWORD * UdmFindCache(UDM_AGENT * Indexer,char *q);
 extern int UdmDeleteURLFromCache(UDM_AGENT *Indexer,int url_id);
-extern int UdmSplitCacheLog(int log);
-extern int UdmPreSplitCacheLog(void);
-extern int UdmClearCacheTree(void);
 
 #endif
