@@ -1341,10 +1341,13 @@ char * UdmUnescapeSgmlStr(char * str){
 				for(e=s+2;(e-s<15)&&(*e<='9')&&(*e>='0');e++);
 				if(*e==';'){
 					int v=atoi(s+2);
-					if(v>=0&&v<=255)*s=(char)v;
-					memmove(s+1,e+1,strlen(e+1)+1);
+					if(v>=0&&v<=255) {
+					  *s=(char)v;
+					} else {
+					  *s = ' ';
+					}
+					memmove(s+1, e+1, strlen(e+1)+1);
 				}
-				s++;
 			}else{
 				for(e=s+1;(e-s<15)&&(((*e<='z')&&(*e>='a'))||((*e<='Z')&&(*e>='A')));e++);
 				if((*e==';')&&(c=UdmSgmlToChar(s+1))){
