@@ -106,14 +106,14 @@ static int InitDB(UDM_AGENT * Indexer){
 
 	if(Indexer->Conf->DBMode==UDM_DBMODE_SINGLE_CRC){
 		char fname[UDMSTRSIZ]="";
-		sprintf(fname,"%s%c%s",UDM_VAR_DIR,UDMSLASH,"dict");
+		sprintf(fname,"%s%s",Indexer->Conf->vardir,"dict");
 		db->crcdict[0]=open(fname,flags,perm);
 		if(db->crcdict[0]<0){
 			sprintf(db->errmsg,"Can't open dict file '%s' (%s)", fname, strerror(errno));
 			db->errcode=1;
 			return(1);
 		}
-		sprintf(fname,"%s%c%s",UDM_VAR_DIR,UDMSLASH,"url.txt");
+		sprintf(fname,"%s%s",Indexer->Conf->vardir,"url.txt");
 		db->url=fopen(fname,fmode);
 		if(!db->url){
 			sprintf(db->errmsg,"Can't open URL file '%s' (%s)", fname, strerror(errno));
@@ -125,7 +125,7 @@ static int InitDB(UDM_AGENT * Indexer){
 		int fnum;
 		char fname[UDMSTRSIZ];
 		for(fnum=1;fnum<MAXMULTI;fnum++){	
-			sprintf(fname,"%s%cdict%02d",UDM_VAR_DIR,UDMSLASH,fnum);
+			sprintf(fname,"%sdict%02d",Indexer->Conf->vardir,fnum);
 			db->crcdict[fnum]=open(fname,flags,perm);
 			if(db->crcdict[fnum]<0){
 				sprintf(db->errmsg,"Can't open dict file '%s' (%s)", fname, strerror(errno));
@@ -133,7 +133,7 @@ static int InitDB(UDM_AGENT * Indexer){
 				return(1);
 			}
 		}
-		sprintf(fname,"%s%c%s",UDM_VAR_DIR,UDMSLASH,"url.txt");
+		sprintf(fname,"%s%s",Indexer->Conf->vardir,"url.txt");
 		db->url=fopen(fname,fmode);
 		if(!db->url){
 			sprintf(db->errmsg,"Can't open URL file '%s' (%s)", fname, strerror(errno));
@@ -142,14 +142,14 @@ static int InitDB(UDM_AGENT * Indexer){
 		}
 	}else{
 		char fname[UDMSTRSIZ];
-		sprintf(fname,"%s%c%s",UDM_VAR_DIR,UDMSLASH,"dict.txt");
+		sprintf(fname,"%s%s",Indexer->Conf->vardir,"dict.txt");
 		db->dict=fopen(fname,fmode);
 		if(!db->dict){
 			sprintf(db->errmsg,"Can't open dict file '%s' (%s)", fname, strerror(errno));
 			db->errcode=1;
 			return(1);
 		}
-		sprintf(fname,"%s%c%s",UDM_VAR_DIR,UDMSLASH,"url.txt");
+		sprintf(fname,"%s%s",Indexer->Conf->vardir,"url.txt");
 		db->url=fopen(fname,fmode);
 		if(!db->url){
 			sprintf(db->errmsg,"Can't open URL file '%s' (%s)", fname, strerror(errno));
@@ -622,6 +622,7 @@ UDM_RESULT * UdmFind(UDM_AGENT * query, char * text){
 	Res=(UDM_RESULT*)malloc(sizeof(UDM_RESULT));
 	Res->Doc=NULL;
 	Res->total_found=0;
+	Res->num_rows=0;
 	strcpy(Res->wordinfo,query->wordinfo);
 
 	/* Return if nothing was found */
@@ -768,6 +769,11 @@ int UdmDBImportAffixes(UDM_AGENT * Indexer, int LCharset){
     return(0);
 }
 
+__INDLIB__ int UdmClearSpell(UDM_AGENT * Indexer){
+    return(0);
+}
+
+
 UDM_CATEGORY * UdmCatPath(UDM_AGENT * Indexer,const char * category){
 	return(NULL);
 }
