@@ -250,7 +250,14 @@ static char *parse_file (UDM_AGENT * Agent,UDM_PARSER * parser,char *buf, size_t
 	arg2pos=strstr(parser->cmd,"$2");
 
 	/* Build temp file names and command line */
-	tmpnam(fn0);strcpy(fn1,fn0);
+#if (WIN32||WINNT)
+	/* Windows generates something strange in tmpnam() */
+	/* So create the name ourself                      */
+	sprintf(fn0, ".\\parser_tmp%d",Agent->handle);
+#else
+	tmpnam(fn0);
+#endif
+	strcpy(fn1,fn0);
 	fnames[0]=strcat(fn0,".in");
 	fnames[1]=strcat(fn1,".out");
 	UdmBuildParamStr(cmd,sizeof(cmd),parser->cmd,fnames,2);
