@@ -41,6 +41,13 @@
 #include <sys/select.h>
 #endif
 
+#ifdef USE_HTTPS
+#include <openssl/crypto.h>
+#include <openssl/x509.h>
+#include <openssl/pem.h>
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+#endif
 
 #include "udm_common.h"
 #include "udm_proto.h"
@@ -104,7 +111,7 @@ int   UdmMirrorGET(UDM_AGENT * Indexer,char * MirrorRoot, char * MirrorHeadersRo
 		strcat(str,".header");
 
 		if ((fheader = open(str, O_RDONLY|UDM_BINARY))>=0) {
-			size = read(fheader, buf, Indexer->maxsize);
+			size = read(fheader, buf, Indexer->Conf->max_doc_size);
 			close(fheader);
 			strcpy(buf + size, "\r\n\r\n");
 			have_headers=1;
@@ -120,7 +127,7 @@ int   UdmMirrorGET(UDM_AGENT * Indexer,char * MirrorRoot, char * MirrorHeadersRo
 	}
 
 	l = strlen(buf);
-	size = read(fbody, buf + l, Indexer->maxsize - l) + l;
+	size = read(fbody, buf + l, Indexer->Conf->max_doc_size - l) + l;
 	close(fbody);
 	return size;
 }
@@ -140,6 +147,10 @@ int   UdmMirrorPUT(UDM_AGENT * Indexer,
 		strcat(str, hostname);
 		strcat(str, path);
 
+#if (WIN32|WINNT)
+		/* Replace slashes to Win style */
+		{char *p; p=str;while(*p){if(*p=='/')*p='\\'; p++;}}
+#endif
 		if(UdmBuild(str, 0755) != 0){
 			sprintf(errstr, "Can't create dir %s", str);
 			return UDM_MIRROR_CANT_BUILD;
@@ -162,6 +173,10 @@ int   UdmMirrorPUT(UDM_AGENT * Indexer,
 		strcat(str, hostname);
 		strcat(str, path);
 
+#if (WIN32|WINNT)
+		/* Replace slashes to Win style */
+		{char *p; p=str;while(*p){if(*p=='/')*p='\\'; p++;}}
+#endif
 		if(UdmBuild(str, 0755) != 0){
 			sprintf(errstr, "Can't create mirror dir %s", str);
 			return UDM_MIRROR_CANT_BUILD;
@@ -261,9 +276,10 @@ static int open_host(UDM_AGENT *Indexer, char *hostname,int port, int timeout)
 
 	net=socket(AF_INET, SOCK_STREAM, 0);
 
-	if(connect_tm(net, (struct sockaddr *)&Indexer->connp->sin, sizeof (struct sockaddr_in),(unsigned int)timeout))
+	if(connect_tm(net, (struct sockaddr *)&Indexer->connp->sin, sizeof (struct sockaddr_in),(unsigned int)timeout)){
+		closesocket(net);
 		return(UDM_NET_CANT_CONNECT);
-
+	}
 	return(net);
 }
 
@@ -394,8 +410,8 @@ int UdmHTTPGet(UDM_AGENT *Indexer,char * header,char *host,int port)
        }else{
            if(FD_ISSET(fd,&sfds))
            {
-		if(nread+buf_size-Indexer->maxsize >=0)
-			buf_size = Indexer->maxsize-nread;
+		if(nread+buf_size-Indexer->Conf->max_doc_size >=0)
+			buf_size = Indexer->Conf->max_doc_size - nread;
 		else
 			buf_size = NET_BUF_SIZE;
 		
@@ -412,7 +428,7 @@ int UdmHTTPGet(UDM_AGENT *Indexer,char * header,char *host,int port)
                    	nread=UDM_NET_TIMEOUT;
                    	break;
                    }
-                   if( nread == Indexer->maxsize )
+                   if( nread == Indexer->Conf->max_doc_size )
                        break;
                }
            }else
@@ -424,8 +440,74 @@ int UdmHTTPGet(UDM_AGENT *Indexer,char * header,char *host,int port)
 }
 #endif
 
+#ifdef USE_HTTPS
+
+#define sslcleanup closesocket(fd); SSL_free (ssl); SSL_CTX_free (ctx)
+
+int UdmHTTPSGet(UDM_AGENT *Indexer,char * header,char *host,int port)
+{
+    int fd;
+    int nread=0;
+    SSL_CTX* ctx;
+    SSL*     ssl;
+    SSL_METHOD *meth;
+
+    /* Connect to HTTPS server */
+    if( (fd=open_host(Indexer,host,port,Indexer->read_timeout)) < 0 )
+       return(fd);
+
+    SSLeay_add_ssl_algorithms();
+    meth = SSLv2_client_method();
+    SSL_load_error_strings();
+    ctx = SSL_CTX_new (meth);
+
+    if ((ctx = SSL_CTX_new (meth))==NULL){
+      sslcleanup;
+      return UDM_NET_ERROR;
+    }
+
+    /* -------------------------------------------------- */
+    /* Now we have TCP connection. Start SSL negotiation. */
+
+    if ((ssl=SSL_new(ctx))==NULL){
+      sslcleanup;
+      closesocket(fd);
+      return UDM_NET_ERROR;
+    }
+
+    SSL_set_fd (ssl, fd);
+
+    if (SSL_connect(ssl)==-1){
+      sslcleanup;
+      return UDM_NET_ERROR;
+    }
+
+    /* Send HTTP request */
+    if ((SSL_write(ssl, header,(int)strlen(header)))==-1){
+      sslcleanup;
+      return UDM_NET_ERROR;
+    }
+
+    /* Get result */
+    nread = SSL_read(ssl, Indexer->buf, (int)(Indexer->Conf->max_doc_size - 1));
+
+    if(nread == -1){
+      sslcleanup;
+      return UDM_NET_ERROR;
+    }
+
+    Indexer->buf[nread] = '\0';
+    SSL_shutdown (ssl);  /* send SSL/TLS close_notify */
+
+    /* Clean up. */
+    sslcleanup;
+    return(nread);
+}
+
+#endif
+
 #ifdef USE_FTP
-int UdmFTPGet(UDM_AGENT * Indexer,char *host,int port, 
+int UdmURLGetFTP(UDM_AGENT * Indexer,char *host,int port, 
 	char *path, char *filename,
 	char *user, char *passwd,
 	int last_mod_time, int checkonly)
@@ -438,7 +520,7 @@ int UdmFTPGet(UDM_AGENT * Indexer,char *host,int port,
 	if (!Indexer->connp->hostname || 
 	    strcmp(Indexer->connp->hostname, host) || 
 	    (Indexer->connp->connected == NET_NOTCONNECTED)){
-		code=ftp_connect(Indexer->Conf,Indexer->connp, host, port, user, passwd, Indexer->read_timeout);
+		code=UdmFTPConnect(Indexer->Conf,Indexer->connp, host, port, user, passwd, Indexer->read_timeout);
 		if (code == -1){
 			if (Indexer->connp->err >0){
 				sprintf(Indexer->buf, "HTTP/1.1 401 OK\r\n\r\n  ");
@@ -452,9 +534,9 @@ int UdmFTPGet(UDM_AGENT * Indexer,char *host,int port,
 	if (Indexer->connp->connected == NET_CONNECTED){
 		/* Make listing */
 		if (!filename){
-			code = ftp_cwd(Indexer->connp, path);
+			code = UdmFTPCwd(Indexer->connp, path);
 			if (code != -1){
-				code = ftp_list(Indexer->connp, Indexer->connp->connp, path, NULL, Indexer->maxsize);
+				code = UdmFTPList(Indexer->connp, Indexer->connp->connp, path, NULL, Indexer->Conf->max_doc_size);
 				if (code == -1){
 					if (Indexer->connp->err >0){
 						sprintf(Indexer->buf, "HTTP/1.1 403 OK\r\n\r\n");
@@ -463,7 +545,7 @@ int UdmFTPGet(UDM_AGENT * Indexer,char *host,int port,
 						size = Indexer->connp->err;
 					}
 				}else{
-					udm_snprintf(Indexer->buf, Indexer->maxsize, "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<html><body>%s</body></html>", 
+					udm_snprintf(Indexer->buf, Indexer->Conf->max_doc_size, "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<html><body>%s</body></html>", 
 						Indexer->connp->connp->buf);
 					size = strlen(Indexer->buf);
 				}
@@ -480,7 +562,7 @@ int UdmFTPGet(UDM_AGENT * Indexer,char *host,int port,
 			len = strlen(path)+strlen(filename);
 			full_path = UdmXmalloc(len+1);
 			udm_snprintf(full_path, len+1, "%s%s", path, filename);
-			last_mod_tm = ftp_mdtm(Indexer->connp,full_path);
+			last_mod_tm = UdmFTPMdtm(Indexer->connp,full_path);
 			if(last_mod_tm == -1 && Indexer->connp->err){
 				if (Indexer->connp->err >0){
 					sprintf(Indexer->buf, "HTTP/1.1 404 OK\r\n\r\n");
@@ -494,12 +576,12 @@ int UdmFTPGet(UDM_AGENT * Indexer,char *host,int port,
 			}else{
 				UdmTime_t2HttpStr(last_mod_tm, buf);
 				if (!checkonly){
-					if (!ftp_get(Indexer->connp, Indexer->connp->connp, full_path,Indexer->maxsize)){
-						udm_snprintf(Indexer->buf, Indexer->maxsize, "HTTP/1.1 200 OK\r\nContent-Type: %s\n\rLast-Modified: %s\r\n\r\n", 
+					if (!UdmFTPGet(Indexer->connp, Indexer->connp->connp, full_path,Indexer->Conf->max_doc_size)){
+						udm_snprintf(Indexer->buf, Indexer->Conf->max_doc_size, "HTTP/1.1 200 OK\r\nContent-Type: %s\n\rLast-Modified: %s\r\n\r\n", 
 							con_type,buf);
 						size = strlen(Indexer->buf);
-						if (Indexer->connp->connp->buf_len+size >= Indexer->maxsize)
-							buf_size = Indexer->maxsize-size;
+						if (Indexer->connp->connp->buf_len+size >= Indexer->Conf->max_doc_size)
+							buf_size = Indexer->Conf->max_doc_size - size;
 						else
 							buf_size = Indexer->connp->connp->buf_len;
 						
@@ -563,7 +645,9 @@ char *lt;
 	size=0;s=Indexer->buf;
 	f=fdopen(fd,"r");f1=fdopen(fd,"w");
 	fgets(str,sizeof(str),f); /* Read hello string */
-
+	fprintf(f1,"mode reader\r\n");fflush(f1);
+	fgets(str,sizeof(str),f); /* Read 2nd hello string*/ 
+	
 	/* All news groups on the server */
 	if((!strcmp(filename,"/"))||(!strcmp(filename,""))){
 		fprintf(f1,"list\r\n");fflush(f1);
@@ -572,7 +656,8 @@ char *lt;
 		while(fgets(str,sizeof(str),f)){
 			if(str[0]=='.')break;
 			if((c=strchr(str,' ')))*c=0;
-			sprintf(UDM_STREND(Indexer->buf),"<A HREF=\"news://%s/%s\"></A>\n",host,str);
+			if(str[0] && ((strlen(Indexer->buf)+strlen(host)+strlen(str)) < Indexer->Conf->max_doc_size - 25))
+			  sprintf(UDM_STREND(Indexer->buf),"<A HREF=\"news://%s/%s\"></A>\n",host,str);
 		}
 		sprintf(UDM_STREND(Indexer->buf),"</BODY></HTML>\n");
 	}else /* List of the articles in one news group */
@@ -599,7 +684,7 @@ char *lt;
 				if((msg_id=UdmGetToken(NULL,"\t\r\n",&lt)));
 				str[0]=0;
 				if(msg_id)sscanf(msg_id,"<%[^>]s>",str);
-				if(str[0] && ((strlen(Indexer->buf)+strlen(host)+strlen(str)) < Indexer->maxsize - 20))
+				if(str[0] && ((strlen(Indexer->buf)+strlen(host)+strlen(str)) < Indexer->Conf->max_doc_size - 25))
 					sprintf(UDM_STREND(Indexer->buf),"<A HREF=\"news://%s/%s\"></A>\n",host,str);
 				/* in the line above, I left 20 bytes of free */
 				/* space at the end of the string for the     */
@@ -634,7 +719,7 @@ char *lt;
 							if(!has_content)strcat(Indexer->buf,"Content-Type: text/plain\r\n");
 						}
 					}
-					if((int)(strlen(Indexer->buf)+strlen(str))<Indexer->maxsize)
+					if((int)(strlen(Indexer->buf)+strlen(str))<Indexer->Conf->max_doc_size)
 						strcat(Indexer->buf,str);
 				}
 			}else{
@@ -684,6 +769,8 @@ char *lt;
 	size=0;s=Indexer->buf;
 	f=fdopen(fd,"r");f1=fdopen(fd,"w");
 	fgets(str,sizeof(str),f); /* Read hello string */
+	fprintf(f1,"mode reader\r\n");fflush(f1);
+	fgets(str,sizeof(str),f); /* Read 2nd hello string*/ 
 
 	/* All news groups on the server */
 	if((!strcmp(filename,"/"))||(!strcmp(filename,""))){
@@ -742,7 +829,7 @@ char *lt;
 							if(!has_content)strcat(Indexer->buf,"Content-Type: text/plain\r\n");
 						}
 					}
-					if((int)(strlen(Indexer->buf)+strlen(str))<Indexer->maxsize)
+					if((int)(strlen(Indexer->buf)+strlen(str))<Indexer->Conf->max_doc_size)
 						strcat(Indexer->buf,str);
 				}
 			}else{
@@ -760,6 +847,191 @@ char *lt;
 
 
 #ifdef USE_FILE
+
+#if (WIN32|WINNT)
+
+static time_t FileTimeToStamp(FILETIME *ft){
+	SYSTEMTIME st;
+	struct tm t;
+
+	FileTimeToSystemTime(ft, &st);
+	t.tm_year	= st.wYear-1900;
+	t.tm_mon	= st.wMonth;
+	t.tm_mday	= st.wDay;
+	t.tm_hour	= st.wHour;
+	t.tm_min	= st.wMinute;
+	t.tm_sec	= st.wSecond;
+
+	return(mktime(&t));
+}
+
+int UdmFILEGet(UDM_AGENT * Indexer,char *header,char* user,char* passwd){
+WIN32_FIND_DATA wfd;
+HANDLE hFind;
+
+WIN32_FILE_ATTRIBUTE_DATA wfad;
+int is_dir;
+int status=0;
+int fd,size,l;
+char filename[UDMSTRSIZ];
+char newfilename[UDMSTRSIZ];
+char openname[UDMSTRSIZ];
+char command[32]="";
+char proto[32]="";
+char host[256]="";
+char *content_type=NULL,*lt,*s;
+time_t ims=0;
+
+	sscanf(header,"%s%s%s",command,filename,proto);
+
+	/* Remember If-Modified-Since timestamp and Host */
+	s=UdmGetToken(header,"\r\n",&lt);
+	while(s){
+		if(!UDM_STRNCASECMP(s,"If-Modified-Since: ")){
+			ims=UdmHttpDate2Time_t(s+19);
+		}
+
+		if(!UDM_STRNCASECMP(s,"Host: ")){
+			strcpy(host,s+6);
+		}
+		s=UdmGetToken(NULL,"\r\n",&lt);
+	}
+
+	if(host[0]){
+		strcpy(newfilename,"\\\\");
+		strcat(newfilename,host);
+		strcat(newfilename,filename);
+	}else{
+		strcpy(newfilename,filename);
+	}
+	UdmUnescapeCGIQuery(openname,newfilename);
+
+	s=openname;
+	while(*s){
+		if(*s=='/')*s='\\';
+		s++;
+	}
+
+	if(!GetFileAttributesEx(openname,GetFileExInfoStandard,&wfad)){
+		int err=GetLastError();
+
+		if(err==ERROR_LOGON_FAILURE){
+			char cname[1024]="\\\\";
+			NETRESOURCE nr;
+			
+			nr.dwType		= RESOURCETYPE_DISK;
+			nr.lpLocalName	= NULL;
+			strcat(cname,host);
+			nr.lpRemoteName	= cname;
+			nr.lpProvider	= NULL;
+
+			err=WNetAddConnection2(&nr, passwd, user, 0);
+
+			switch(err){
+				case NO_ERROR: break;
+				case ERROR_LOGON_FAILURE:
+				default: 
+					status=403; 
+					break;
+			}
+			if(!GetFileAttributesEx(openname,GetFileExInfoStandard,&wfad)) status=403;
+		}else{
+			switch(err){
+				case ERROR_FILE_NOT_FOUND:
+				case ERROR_PATH_NOT_FOUND:	status=404; break;
+				case ERROR_ACCESS_DENIED:	status=403; break;
+				default:					status=500; break;
+			}
+		}
+		if(status){
+			sprintf(Indexer->buf,"HTTP/1.0 %d %s\r\n\r\n",status,UdmHTTPErrMsg(status));
+			return(strlen(Indexer->buf));
+		}
+	}
+
+
+	is_dir=wfad.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY;
+
+	if(is_dir){
+		if(filename[strlen(filename)-1]!='/') strcat(filename,"/");
+		strcat(openname,"\\*");
+		hFind=FindFirstFile(openname, &wfd);
+		if(hFind==INVALID_HANDLE_VALUE){
+			status=500;
+			sprintf(Indexer->buf,"HTTP/1.0 %d %s\r\n\r\n",status,UdmHTTPErrMsg(status));
+			return(strlen(Indexer->buf));
+		}
+		strcpy(Indexer->buf,"HTTP/1.0 200 OK\r\nContent-type: text/html\r\n\r\n<HTML><BODY>\n");
+		do{
+			char escaped_name[UDMSTRSIZ]="";
+			char *src,*e;
+
+			if(!strcmp(wfd.cFileName,"..") || !strcmp(wfd.cFileName,".")) continue;
+
+			e=escaped_name;
+			for(src=wfd.cFileName;*src;src++){
+				if(strchr(" %&<>+[](){}/?#'\"\\;,",*src)){
+					sprintf(e,"%%%X",(int)*src);
+					e+=3;
+				}else{
+					*e=*src;
+					e++;
+				}
+			}
+
+			*e=0;
+			is_dir=wfd.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY;
+			if(host[0]){
+				sprintf(UDM_STREND(Indexer->buf),"<A HREF=\"file://%s%s%s%s\">%s</A>\n",
+					host,filename,escaped_name,is_dir?"/":"",escaped_name);
+			}else{
+				sprintf(UDM_STREND(Indexer->buf),"<A HREF=\"file:%s%s%s\">%s</A>\n",
+					filename,escaped_name,is_dir?"/":"",escaped_name);
+			}
+		}while(FindNextFile(hFind, &wfd));
+
+		strcpy(UDM_STREND(Indexer->buf),"</BODY><HTML>\n");
+		FindClose(hFind);
+		return(strlen(Indexer->buf));
+	}else{
+		/* Lets compare last modification date */
+
+		if(ims>=FileTimeToStamp(&wfad.ftLastWriteTime)){
+			status=304;
+			sprintf(Indexer->buf,"HTTP/1.0 %d %s\r\n\r\n",status,UdmHTTPErrMsg(status));
+			size=strlen(Indexer->buf);
+			return(size);
+		}
+
+		if((fd=open(openname,O_RDONLY|UDM_BINARY))<0){
+			int err=errno;
+			switch(err){
+				case ENOENT: status=404;break;
+				case EACCES: status=403;break;
+				default: status=1;
+			}
+			sprintf(Indexer->buf,"HTTP/1.0 %d %s %s\r\n\r\n",status,UdmHTTPErrMsg(status),strerror(err));
+			return(strlen(Indexer->buf));
+		}
+		sprintf(Indexer->buf,"HTTP/1.0 200 OK\r\n");
+		if((content_type=UdmContentType(Indexer->Conf,filename)))
+			sprintf(UDM_STREND(Indexer->buf),"Content-Type: %s\r\n",content_type);
+
+		
+		strcpy(UDM_STREND(Indexer->buf),"Last-Modified: ");
+		UdmTime_t2HttpStr(FileTimeToStamp(&wfad.ftLastWriteTime), UDM_STREND(Indexer->buf));
+		strcpy(UDM_STREND(Indexer->buf),"\r\n");
+
+		sprintf(UDM_STREND(Indexer->buf),"\r\n");
+		l=strlen(Indexer->buf);
+		size = read(fd, Indexer->buf + l, Indexer->Conf->max_doc_size - l) + l;
+		close(fd);
+		return(size);
+	}
+}
+
+#else
+
 int UdmFILEGet(UDM_AGENT * Indexer,char *header){
 int fd,size,l,status=0,is_dir;
 struct stat sb,sb1;
@@ -768,6 +1040,7 @@ struct dirent *rec;
 char filename[UDMSTRSIZ];
 char newfilename[UDMSTRSIZ];
 char openname[UDMSTRSIZ];
+char mystatname[UDMSTRSIZ];
 char command[32]="";
 char proto[32]="";
 char *content_type=NULL,*lt,*s;
@@ -786,16 +1059,9 @@ time_t ims=0;
 		s=UdmGetToken(NULL,"\r\n",&lt);
 	}
 
-	s=openname;
-#if (WIN32|WINNT)
-	while(*s){
-		if(*s=='/')*s='\\';
-		s++;
-	}
-	if ((strlen(openname)>3)&&(openname[strlen(openname)-1]=='\\'))
-		openname[strlen(openname)-1]='\0';
-#endif
-	if(stat(openname,&sb)){
+	strcpy(mystatname,openname);
+	
+	if(stat(mystatname,&sb)){
 		switch(errno){
 			case ENOENT: status=404;break; /* Not found */
 			case EACCES: status=403;break; /* Forbidden*/
@@ -885,9 +1151,12 @@ time_t ims=0;
 
 		sprintf(UDM_STREND(Indexer->buf),"\r\n");
 		l=strlen(Indexer->buf);
-		size=read(fd,Indexer->buf+l,Indexer->maxsize-l)+l;
+		size = read(fd, Indexer->buf + l, Indexer->Conf->max_doc_size - l) + l;
 		close(fd);
 		return(size);
 	}
 }
+
+#endif
+
 #endif
