@@ -26,15 +26,21 @@ int UdmFreeRobots(UDM_ROBOTS * Robots){
 	Robots->nrobots = 0;
 	return(0);
 }
+	
 UDM_ROBOT * UdmFindRobots(UDM_ENV * Conf,UDM_URL * URL){
 	size_t i;
+	char path[UDM_URLSIZE];
+
+	strcpy(path, URL->path);
+	strcat(path, URL->filename);
 
 	for(i=0;i<Conf->Robots.nrobots;i++){
-		if(!strcmp(URL->hostinfo,Conf->Robots.robots[i].hostinfo)&&!strncmp(URL->path,Conf->Robots.robots[i].path,strlen(Conf->Robots.robots[i].path)))
-			return(&Conf->Robots.robots[i]);
+		if(!strcmp(URL->hostinfo,Conf->Robots.robots[i].hostinfo)&&!strncmp(path,Conf->Robots.robots[i].path,strlen(Conf->Robots.robots[i].path)))
+		return(&Conf->Robots.robots[i]);
 	}
 	return(NULL);
 }
+
 int UdmParseRobots(UDM_AGENT *Indexer,UDM_ROBOTS * Robots,char *content,char *hostinfo){
 	int myrule=0;
 	char *s,*e,*lt;
