@@ -240,6 +240,8 @@ UDM_SERVER * UdmFindServer(UDM_ENV * Conf,char *url, char * aliastr){
 		if(!strcmp(robots,"/robots.txt")){
 			robots=strdup(url);
 			robots[strlen(url)-10]='\0';
+		}else{
+			robots=NULL;
 		}
 	}
 
@@ -295,7 +297,7 @@ UDM_SERVER * UdmFindServer(UDM_ENV * Conf,char *url, char * aliastr){
 					if(Conf->Server[i].match_type&UDM_SERVER_CS){
 						res=UDM_STRNCMP(Conf->Server[i].url,robots);
 					}else{
-						res=UDM_STRNCASECMP(Conf->Server[i].url,robots);
+						res=UDM_STRNCMP(Conf->Server[i].url,robots);
 					}
 				}else{
 					if(Conf->Server[i].follow==UDM_FOLLOW_NO){
@@ -308,7 +310,7 @@ UDM_SERVER * UdmFindServer(UDM_ENV * Conf,char *url, char * aliastr){
 						if(Conf->Server[i].match_type&UDM_SERVER_CS){
 							res=UDM_STRNCMP(url,Conf->Server[i].url);
 						}else{
-							res=UDM_STRNCASECMP(url,Conf->Server[i].url);
+							res=UDM_STRNCMP(url,Conf->Server[i].url);
 						}
 					}
 				}
