@@ -100,7 +100,7 @@ int UdmAddSpell(UDM_ENV * Conf,const char * word,const char *flag,const char *la
 	Conf->Spell[Conf->nspell].word=strdup(word);
 	strncpy(Conf->Spell[Conf->nspell].flag,flag,10);
 	strncpy(Conf->Spell[Conf->nspell].lang,lang,2);
-	Conf->Spell[Conf->nspell].lang[3] = 0;
+	Conf->Spell[Conf->nspell].lang[2] = 0;
 	Conf->nspell++;
 	return(0);
 }
@@ -120,7 +120,7 @@ __INDLIB__ int UdmImportDictionary(UDM_ENV * Conf,const char *lang,const char *f
 			*s=0;
 			s++;flag=s;
 			while(*s){
-				if((*s>='A')&&(*s<='Z'))s++;
+				if(((*s>='A')&&(*s<='Z'))||((*s>='a')&&(*s<='z')))s++;
 				else{
 					*s=0;
 					break;
@@ -152,23 +152,24 @@ __INDLIB__ int UdmImportDictionary(UDM_ENV * Conf,const char *lang,const char *f
 __INDLIB__ int UdmDBImportDictionary(UDM_AGENT * Indexer,const char *lang,const char *filename, int dump){
 	unsigned char str[BUFSIZ];
 	unsigned char *s;
-	const unsigned char *flag=NULL;
+	const unsigned char *flag="";
 	int rej=0;
 	int imp=0;
 	FILE *dict;
 
 	if(!(dict=fopen(filename,"r")))return(1);
 	while(fgets(str,sizeof(str),dict)){
-		flag=NULL;
 		if((s=strchr(str,'/'))){
 			*s=0;s++;flag=s;
 			while(*s){
-				if(*s>='A'&&*s<='Z')s++;
+				if((*s>='A'&&*s<='Z')||(*s>='a'&&*s<='z'))s++;
 				else{
 					*s=0;
 					break;
 				}
 			}
+		} else {
+			flag="";
 		}
 		
 		UdmTolower(str,Indexer->Conf->local_charset);
@@ -181,11 +182,12 @@ __INDLIB__ int UdmDBImportDictionary(UDM_AGENT * Indexer,const char *lang,const
 		}
 		
 		if(dump){
-			printf("INSERT INTO spell (word,flag,lang) VALUES ('%s','%s','%s');\n",str,flag?flag:(const unsigned char*)"",lang);
+			printf("INSERT INTO spell (word,flag,lang) VALUES ('%s','%s','%s');\n",str,flag,lang);
 		}else{
 			if(UdmInsertSpell(Indexer,flag,lang,str)) {
 				rej++;
 				printf("InsertSpell %d: %s\n",rej,UdmDBErrorMsg(Indexer->db));
+			printf("INSERT INTO spell (word,flag,lang) VALUES ('%s','%s','%s');\n",str,flag,lang);
 			}
 			else
 				imp++;
@@ -198,6 +200,7 @@ __INDLIB__ int UdmDBImportDictionary(UDM_AGENT * Indexer,const char *lang,const
 	return(0);
 }
 
+
 static UDM_SPELL * UdmFindWord(UDM_AGENT * Indexer, const char *word, int affixflag) {
 int l,c,r,resc,resl,resr, nlang = Indexer->spellang, li, li_from, li_to, i;
 
@@ -218,17 +221,17 @@ int l,c,r,resc,resl,resr, nlang = Indexer->spellang, li, li_from, li_to, i;
 		    c = (l + r) >> 1;
 		    resc = strcmp(Indexer->Conf->Spell[c].word, word);
 		    if( (resc == 0) && 
-			((affixflag == NULL) || (strchr(Indexer->Conf->Spell[c].flag, affixflag) != NULL)) ) {
+			((affixflag == 0) || (strchr(Indexer->Conf->Spell[c].flag, affixflag) != NULL)) ) {
 		      return(&Indexer->Conf->Spell[c]);
 		    }
 		    resl = strcmp(Indexer->Conf->Spell[l].word, word);
 		    if( (resl == 0) && 
-			((affixflag == NULL) || (strchr(Indexer->Conf->Spell[l].flag, affixflag) != NULL)) ) {
+			((affixflag == 0) || (strchr(Indexer->Conf->Spell[l].flag, affixflag) != NULL)) ) {
 		      return(&Indexer->Conf->Spell[l]);
 		    }
 		    resr = strcmp(Indexer->Conf->Spell[r].word, word);
 		    if( (resr == 0) && 
-			((affixflag == NULL) || (strchr(Indexer->Conf->Spell[r].flag, affixflag) != NULL)) ) {
+			((affixflag == 0) || (strchr(Indexer->Conf->Spell[r].flag, affixflag) != NULL)) ) {
 		      return(&Indexer->Conf->Spell[r]);
 		    }
 		    if(resc < 0){
@@ -352,7 +355,7 @@ FILE *affix;
 		    switch(i){
 		    case 3:break;
 		    case 2:
-			    if(*find!='-'){
+			    if(*find != '\0'){
 				    strcpy(repl,find);
 				    strcpy(find,"");
 			    }
@@ -390,7 +393,11 @@ __INDLIB__ void UdmSortDictionary(UDM_ENV * Conf){
   int  j, CurLet = -1, Let;size_t i;
   char *CurLang = NULL;
 
+#ifdef HAVE_MERGESORT
+        mergesort((void*)Conf->Spell,Conf->nspell,sizeof(UDM_SPELL),cmpspell);
+#else
         qsort((void*)Conf->Spell,Conf->nspell,sizeof(UDM_SPELL),cmpspell);
+#endif
 	for(i = 0; i < Conf->nspell; i++) {
 	  if (CurLang == NULL || strncmp(CurLang, Conf->Spell[i].lang, 2) != 0) {
 	    CurLang = Conf->Spell[i].lang;
@@ -462,7 +469,7 @@ __INDLIB__ void UdmSortAffixes(UDM_ENV * Conf) {
 	CurLetS = -1;
       }
       if (cl < 0) continue; /* we have affixes without spell for this lang */
-      Let = (int)(Affix->repl[Affix->replen-1]) & 255;
+      Let = (Affix->replen) ? (int)(Affix->repl[Affix->replen-1]) & 255 : 0;
       if (CurLetS != Let) {
 	Conf->SuffixTree[cl].Left[Let] = i;
 	CurLetS = Let;
@@ -476,6 +483,7 @@ static char * CheckSuffix(const char *word, size_t len, UDM_AFFIX *Affix, int *r
   regmatch_t subs[1];
   char newword[2*MAXNORMLEN] = "";
   int err;
+  int curlang, curspellang;
   
   *res = strbncmp(word, Affix->repl, Affix->replen);
   if (*res < 0) {
@@ -501,8 +509,15 @@ static char * CheckSuffix(const char *word, size_t len, UDM_AFFIX *Affix, int *r
 
     if((curspell=UdmFindWord(Indexer, newword, Affix->flag))){
     /*  if(strchr(curspell->flag,Affix->flag)){*/
+      
+      curlang = Indexer->curlang;
+      curspellang = Indexer->spellang;
+      UdmSelectLang(Indexer, curspell->lang);
+      Indexer->lang[Indexer->curlang].count++;
+      Indexer->curlang = curlang;
+      Indexer->spellang = curspellang;
 	
-        return strdup(newword);
+      return strdup(newword);
     /*  }*/
     }
   }
@@ -515,9 +530,10 @@ static int CheckPrefix(const char *word, size_t len, UDM_AFFIX *Affix, UDM_AGENT
 		char **forms, char ***cur ) {
   regmatch_t subs[NS];
   char newword[2*MAXNORMLEN] = "";
-  int err, ls,cs,rs, lres,rres,cres, res;
+  int err, ls, rs, lres,rres, res;
   size_t newlen;
   UDM_AFFIX *CAffix = Indexer->Conf->Affix;
+  int curlang, curspellang;
   
   res = strncmp(word, Affix->repl, Affix->replen);
   if (res != 0) {
@@ -539,6 +555,12 @@ static int CheckPrefix(const char *word, size_t len, UDM_AFFIX *Affix, UDM_AGENT
     UDM_SPELL * curspell;
 
     if((curspell=UdmFindWord(Indexer, newword, Affix->flag))){
+      curlang = Indexer->curlang;
+      curspellang = Indexer->spellang;
+      UdmSelectLang(Indexer, curspell->lang);
+      Indexer->lang[Indexer->curlang].count++;
+      Indexer->curlang = curlang;
+      Indexer->spellang = curspellang;
       if ((*cur - forms) < (MAX_NORM-1)) {
 	**cur =  strdup(newword);
 	(*cur)++; **cur = NULL;
@@ -548,35 +570,20 @@ static int CheckPrefix(const char *word, size_t len, UDM_AFFIX *Affix, UDM_AGENT
     ls = Indexer->Conf->SuffixTree[li].Left[pi];
     rs = Indexer->Conf->SuffixTree[li].Right[pi];
     while (ls >= 0 && ls <= rs) {
-      cs = (ls + rs) >> 1; /*cs = ls;*/
-      if ((*cur - forms) < (MAX_NORM-1)) {
-	**cur = CheckSuffix(newword, newlen, &CAffix[cs], &cres, Indexer);
-	if (**cur) {
-	  (*cur)++; **cur = NULL;
-	}
-      }
-      if ( (ls < cs) && ((*cur - forms) < (MAX_NORM-1)) ) {
+      if (  ((*cur - forms) < (MAX_NORM-1)) ) {
 	**cur = CheckSuffix(newword, newlen, &CAffix[ls], &lres, Indexer);
 	if (**cur) {
 	  (*cur)++; **cur = NULL;
 	}
       }
-      if ( (rs > cs) && ((*cur - forms) < (MAX_NORM-1)) ) {
+      if ( (rs > ls) && ((*cur - forms) < (MAX_NORM-1)) ) {
 	**cur = CheckSuffix(newword, newlen, &CAffix[rs], &rres, Indexer);
 	if (**cur) {
 	  (*cur)++; **cur = NULL;
 	}
       }
-      if (cres < 0) {
-	rs = cs - 1;
-	ls++;
-      } else if (rres > 0) {
-	ls = cs + 1;
-	rs--;
-      } else {
-	ls++;
-	rs--;
-      }
+      ls++;
+      rs--;
     }
   }
   return 0;
@@ -589,8 +596,10 @@ size_t len;
 char ** forms;
 char **cur;
 UDM_AFFIX * Affix;
-int ri, pi, lp, rp, cp, ls, rs, cs, nlang = Indexer->spellang;
-int li, li_from, li_to, lres, cres, rres;
+int ri, pi, ipi, lp, rp, cp, ls, rs, nlang = Indexer->spellang;
+int li, li_from, li_to, lres, rres, cres = 0;
+ int curlang, curspellang;
+  UDM_SPELL *spell;
 
 	len=strlen(word);
 	if (len < Indexer->Conf->min_word_len 
@@ -613,76 +622,71 @@ int li, li_from, li_to, lres, cres, rres;
 	Affix=(UDM_AFFIX*)Indexer->Conf->Affix;
 
 	/* Check that the word itself is normal form */
-	if(UdmFindWord(Indexer, word, NULL)){
+	if((spell = UdmFindWord(Indexer, word, 0))){
 		*cur=strdup(word);
 		cur++;*cur=NULL;
+		curlang = Indexer->curlang;
+		curspellang = Indexer->spellang;
+		UdmSelectLang(Indexer, spell->lang);
+		Indexer->lang[Indexer->curlang].count++;
+		Indexer->curlang = curlang;
+		Indexer->spellang = curspellang;
 	}
 
 	/* Find all other NORMAL forms of the 'word' */
 
-	for (li = li_from; li < li_to; li++) {
-	  /* check prefix */
-	  lp = Indexer->Conf->PrefixTree[li].Left[ri];
-	  rp = Indexer->Conf->PrefixTree[li].Right[ri];
-	  while (lp >= 0 && lp <= rp) {
-	    cp = (lp + rp) >> 1;
-	    if ((cur - forms) < (MAX_NORM-1)) {
-	      cres = CheckPrefix(word, len, &Affix[cp], Indexer, li, pi, forms, &cur);
-	    }
-	    if ((lp < cp) && ((cur - forms) < (MAX_NORM-1)) ) {
-	      lres = CheckPrefix(word, len, &Affix[lp], Indexer, li, pi, forms, &cur);
-	    }
-	    if ( (rp > cp) && ((cur - forms) < (MAX_NORM-1)) ) {
-	      rres = CheckPrefix(word, len, &Affix[rp], Indexer, li, pi, forms, &cur);
-	    }
-	    if (cres < 0) {
-	      rp = cp - 1;
-	      lp++;
-	    } else if (rres > 0) {
-	      lp = cp + 1;
-	      rp--;
-	    } else {
-	      lp++;
-	      rp--;
-	    }
-	  }
+	for (ipi = 0; ipi <= pi; ipi += pi) {
 
-	  /* check suffix */
-	  ls = Indexer->Conf->SuffixTree[li].Left[pi];
-	  rs = Indexer->Conf->SuffixTree[li].Right[pi];
-	  while (ls >= 0 && ls <= rs) {
-	    cs = (ls + rs) >> 1;  /*cs = ls;*/
-	    if ((cur - forms) < (MAX_NORM-1)) {
-	      *cur = CheckSuffix(word, len, &Affix[cs], &cres, Indexer);
-	      if (*cur) {
-		cur++; *cur = NULL;
+	  for (li = li_from; li < li_to; li++) {
+	    /* check prefix */
+	    lp = Indexer->Conf->PrefixTree[li].Left[ri];
+	    rp = Indexer->Conf->PrefixTree[li].Right[ri];
+	    while (lp >= 0 && lp <= rp) {
+	      cp = (lp + rp) >> 1;
+	      cres = 0;
+	      if ((cur - forms) < (MAX_NORM-1)) {
+		cres = CheckPrefix(word, len, &Affix[cp], Indexer, li, ipi, forms, &cur);
+	      }
+	      if ((lp < cp) && ((cur - forms) < (MAX_NORM-1)) ) {
+		lres = CheckPrefix(word, len, &Affix[lp], Indexer, li, ipi, forms, &cur);
+	      }
+	      if ( (rp > cp) && ((cur - forms) < (MAX_NORM-1)) ) {
+		rres = CheckPrefix(word, len, &Affix[rp], Indexer, li, ipi, forms, &cur);
+	      }
+	      if (cres < 0) {
+		rp = cp - 1;
+		lp++;
+	      } else if (cres > 0) {
+		lp = cp + 1;
+		rp--;
+	      } else {
+		lp++;
+		rp--;
 	      }
 	    }
-	    if ( (ls < cs) && ((cur - forms) < (MAX_NORM-1)) ) {
-	      *cur = CheckSuffix(word, len, &Affix[ls], &lres, Indexer);
-	      if (*cur) {
-		cur++; *cur = NULL;
+
+	    /* check suffix */
+	    ls = Indexer->Conf->SuffixTree[li].Left[ipi];
+	    rs = Indexer->Conf->SuffixTree[li].Right[ipi];
+	    while (ls >= 0 && ls <= rs) {
+	      if (  ((cur - forms) < (MAX_NORM-1)) ) {
+		*cur = CheckSuffix(word, len, &Affix[ls], &lres, Indexer);
+		if (*cur) {
+		  cur++; *cur = NULL;
+		}
 	      }
-	    }
-	    if ( (rs > cs) && ((cur - forms) < (MAX_NORM-1)) ) {
-	      *cur = CheckSuffix(word, len, &Affix[rs], &rres, Indexer);
-	      if (*cur) {
-		cur++; *cur = NULL;
+	      if ( (rs > ls) && ((cur - forms) < (MAX_NORM-1)) ) {
+		*cur = CheckSuffix(word, len, &Affix[rs], &rres, Indexer);
+		if (*cur) {
+		  cur++; *cur = NULL;
+		}
 	      }
-	    }
-	    if (cres < 0) {
-	      rs = cs - 1;
-	      ls++;
-	    } else if (rres > 0) {
-	      ls = cs + 1;
-	      rs--;
-	    } else {
 	      ls++;
 	      rs--;
-	    }
-	  } /* end while */
+	    } /* end while */
 	  
-	} /* for li */
+	  } /* for li */
+	} /* for ipi */
 
 	if(cur==forms){
 		free(forms);
@@ -756,20 +760,20 @@ int UdmLoadSpellFromServer(UDM_ENV *Conf, char *hostname) {
   }
   
   if(connect(s, (struct sockaddr *)&server_addr, sizeof(server_addr)) == -1) {
-    close(s);
+    closesocket(s);
     return 3;
   }
 
   send (s, hello, 4, 0);
 
   if (recvall(s, &Conf->nLang, sizeof(Conf->nLang), MSG_WAITALL) < 0) {
-    close(s);
+    closesocket(s);
     return 4;
   }
 
   
   if (recvall(s, (void *)&Conf->naffixes, sizeof(Conf->naffixes), MSG_WAITALL) < 0) {
-    close(s);
+    closesocket(s);
     return 4;
   }
   Conf->maffixes = Conf->naffixes;
@@ -777,29 +781,29 @@ int UdmLoadSpellFromServer(UDM_ENV *Conf, char *hostname) {
   Affix = (UDM_AFFIX *)Conf->Affix;
   for(i = 0; i < Conf->naffixes; i++) {
     if (recvall(s, &Affix[i], sizeof(UDM_AFFIX), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
     regcomp(&Affix[i].reg, Affix[i].mask, REG_EXTENDED|REG_ICASE);
   }
   if (recvall(s, (void *)&Conf->nspell, sizeof(Conf->nspell), MSG_WAITALL) < 0) {
-    close(s);
+    closesocket(s);
     return 4;
   }
   Conf->mspell = Conf->nspell;
   Conf->Spell = (UDM_SPELL *) UdmXmalloc(Conf->nspell * sizeof(UDM_SPELL));
   for(j = 0; j < Conf->nspell; j++) {
     if (recvall(s, &Conf->Spell[j], sizeof(UDM_SPELL), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
     if (recvall(s, &wlen, sizeof(int), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
     Conf->Spell[j].word = (char *)UdmXmalloc(wlen + 1);
     if (recvall(s, Conf->Spell[j].word, wlen, MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
     Conf->Spell[j].word[wlen] = 0;
@@ -807,20 +811,20 @@ int UdmLoadSpellFromServer(UDM_ENV *Conf, char *hostname) {
 
   for(i = 0; i < Conf->nLang; i++) {
     if (recvall(s, &Conf->PrefixTree[i], sizeof(Tree_struct), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
     if (recvall(s, &Conf->SuffixTree[i], sizeof(Tree_struct), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
     if (recvall(s, &Conf->SpellTree[i], sizeof(Tree_struct), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       return 4;
     }
   }
 
-  close(s);
+  closesocket(s);
 
   return 0;
 }
@@ -832,7 +836,7 @@ __INDLIB__ char ** UdmNormalizeWordFromServer(UDM_AGENT *Indexer, const char *wo
   int s;
   size_t len;
   const char *hello = "CLIL\0";
-  const char  *lang = "  ";
+  const char  *lang = "\0  ";
   char buf[1024];
 
   len=strlen(word);
@@ -855,7 +859,7 @@ __INDLIB__ char ** UdmNormalizeWordFromServer(UDM_AGENT *Indexer, const char *wo
   }
   
   if(connect(s, (struct sockaddr *)&server_addr, sizeof(server_addr)) == -1) {
-    close(s);
+    closesocket(s);
     return NULL;
   }
 
@@ -869,13 +873,13 @@ __INDLIB__ char ** UdmNormalizeWordFromServer(UDM_AGENT *Indexer, const char *wo
 
   do {
     if (recvall(s, &len, sizeof(len), MSG_WAITALL) < 0) {
-      close(s);
+      closesocket(s);
       free(forms);
       return NULL;
     }
     if (len > 0) {
       if (recvall(s, buf, len, MSG_WAITALL) < 0) {
-	close(s);
+	closesocket(s);
 	free(forms);
 	return NULL;
       }
@@ -887,7 +891,7 @@ __INDLIB__ char ** UdmNormalizeWordFromServer(UDM_AGENT *Indexer, const char *wo
 
   } while (len > 0);
 
-  close(s);
+  closesocket(s);
 
   if(cur==forms){
     free(forms);
