@@ -23,27 +23,32 @@ http://search.mnogo.ru (C) 1998-2001, LavTech Corp.\n\
 Usage: splitter [OPTIONS]\n\
 \n\
 Options are:\n\
+  -w /path      choose alternative working /var directory\n\
   -p            prepare cachelogd logs for splitter\n\
   -f xxx        start at xxx.log, where xxx is a hex number \n\
   -t xxx        stop  at xxx.log, where xxx is a hex number\n\
   -h,-?         print this help page and exit\n\
 \n\
-Please mail bug reports and suggestions to <udmsearch@search.udm.net>.\n",UdmVersion());
+Please mail bug reports and suggestions to <general@mnogosearch.org>.\n",UdmVersion());
 
 	return;
 }
 
 
 int main(int argc,char **argv){
-int ch,log;
-int from=0;
-int to=0xFFF;
+	int ch,log;
+	int from=0;
+	int to=0xFFF;
+	int presplit=0;
+	UDM_ENV * Env;
 
-	while ((ch = getopt(argc, argv, "pt:f:h?")) != -1){
+	Env=UdmAllocEnv();
+
+	while ((ch = getopt(argc, argv, "pt:f:w:h?")) != -1){
+		printf("ch=%c\n",ch);
 		switch (ch) {
 			case 'p':
-				UdmPreSplitCacheLog();
-				return 0;
+				presplit=1;
 				break;
 			case 'f':
 				sscanf(optarg,"%x",&from);
@@ -55,6 +60,9 @@ int to=0xFFF;
 				if(to<0)to=0;
 				if(to>0xFFF)to=0xFFF;
 				break;
+			case 'w':
+				sprintf(Env->vardir,"%s%s",optarg,UDMSLASHSTR);
+				break;
 			case 'h':
 			case '?':
 			default:
@@ -70,9 +78,12 @@ int to=0xFFF;
 		return 1;
 	}
 
-	for(log=from;log<=to;log++){
-		UdmSplitCacheLog(log);
+	if(presplit){
+		UdmPreSplitCacheLog(Env);
+	}else{
+		for(log=from;log<=to;log++){
+			UdmSplitCacheLog(Env,log);
+		}
 	}
-	
 	return 0;
 }
