@@ -857,7 +857,7 @@ time_t Udm_dp2time_t(char * time_str){
 				t+=i*60*60*24*30; flag=1; break;
 			case 'y': /* years */
 				t+=i*60*60*24*365; flag=1; break;
-			case NULL: /* end of string */
+			case '\0': /* end of string */
 				if (flag==1)
 					return -1;
 				else{
@@ -908,12 +908,17 @@ void UdmTime_t2HttpStr(time_t t, char *str){
     if (t==0)
 	*str='\0';
     else{
-	/* FIXME: I suspect that gmtime IS NOT REENTRABLE!
-	 */
+	/* FIXME: I suspect that gmtime IS NOT REENTRANT */
+
 	tim=gmtime(&t);
+
+#if (WIN32||WINNT)
+	if (!strftime(str, UDM_MAXTIMESTRLEN, "%a, %d %b %Y %H:%M:%S GMT", tim))
+#else
 	if (!strftime(str, UDM_MAXTIMESTRLEN, "%a, %d %b %Y %H:%M:%S %Z", tim))
-	    *str='\0';
-	/* end of BAD code */
+#endif
+		*str='\0';
+	/* /FIXME end of BAD code */
     }
 }
 
@@ -1291,6 +1296,60 @@ int udm_snprintf(char *buf, size_t len, const char *fmt, ...){
 
 
 
+#if (WIN32||WINNT)
+int UdmBuild(char *path, int omode) {
+	struct stat sb;
+	int last, retval, ret_code;
+	char *p;
+
+	p = path;
+	retval = 0;
+	if (p[0] == UDMSLASH)		/* Skip leading slashes. */
+		++p;
+	for (last = 0; !last ; ++p) {
+		if (p[0] == '\0')
+			last = 1;
+		else
+		if (p[0] != UDMSLASH)
+			continue;
+		*p = '\0';
+		if (p[1] == '\0')
+			last = 1;
+
+		if(path[strlen(path)-1]==':'){
+			char buf[5*1024];
+
+			sprintf(buf,"%s",path);
+			strcat(buf, "\\");
+			ret_code=stat(buf, &sb);
+		}else{
+			ret_code=stat(path, &sb);
+		}
+
+		if (ret_code) {
+			if (errno != ENOENT || mkdir(path) < 0 ){
+				/* warn("%s", path); */
+				retval = 1;
+				break;
+			}
+		}
+		else if ((sb.st_mode & S_IFMT) != S_IFDIR) {
+			if (last)
+				errno = EEXIST;
+			else
+				errno = ENOTDIR;
+			/* warn("%s", path); */
+			retval = 1;
+			break;
+		}
+		if (!last)
+			*p = UDMSLASH;
+	}
+	return (retval);
+}
+
+#else
+
 int UdmBuild(char *path, int omode) {
 	struct stat sb;
 	mode_t numask, oumask;
@@ -1324,11 +1383,7 @@ int UdmBuild(char *path, int omode) {
 			 * instead of doing chmod's.
 			 */
 			oumask = umask(0);
-#if (WIN32||WINNT)
-			numask = oumask;
-#else
 			numask = oumask & ~(S_IWUSR | S_IXUSR);
-#endif
 			(void)umask(numask);
 			first = 0;
 		}
@@ -1336,12 +1391,8 @@ int UdmBuild(char *path, int omode) {
 			(void)umask(oumask);
 		if (stat(path, &sb)) {
 			if (errno != ENOENT ||
-#if (WIN32||WINNT)
-			    mkdir(path) < 0
-#else
 			    mkdir(path, last ? omode :
 				  S_IRWXU | S_IRWXG | S_IRWXO) < 0
-#endif
 				 ){
 				/* warn("%s", path); */
 				retval = 1;
@@ -1365,7 +1416,7 @@ int UdmBuild(char *path, int omode) {
 	return (retval);
 }
 
-
+#endif
 
 #define udm_tolower(c) (tolower(((unsigned char)(c))))
 
@@ -1427,7 +1478,13 @@ char * UdmBuildParamStr(char * dst,size_t len,const char * src,char ** argv,size
 				size_t arglen;
 				arglen=strlen(argv[argn-1]);
 				if(arglen+curlen+1<len){
+#if (WIN32||WINNT)
+					strcpy(d,"\"");
+					strcat(d,argv[argn-1]);
+					strcat(d,"\"");
+#else
 					strcpy(d,argv[argn-1]);
+#endif
 					d+=strlen(d);
 					curlen+=arglen;
 				}else{
@@ -1438,6 +1495,8 @@ char * UdmBuildParamStr(char * dst,size_t len,const char * src,char ** argv,size
 			while(*s>='0'&&*s<='9')
 				s++;
 		}else
+#if (WIN32||WINNT)
+#else
 		if(*s=='\\'){
 			s++;
 			if(*s){
@@ -1451,7 +1510,9 @@ char * UdmBuildParamStr(char * dst,size_t len,const char * src,char ** argv,size
 					break;
 				}
 			}
-		}else{
+		}else
+#endif
+		{
 			if(curlen+2<len){
 				*d=*s;
 				s++;
